@@ -1,23 +1,41 @@
+# coding=utf-8
+
 import base64
+import random
+import sys
 
 from . import documents, results, html_paths, images
 from .html_generation import HtmlGenerator, satisfy_html_path
 
 
-def convert_document_element_to_html(element, style_map=None, convert_image=None, convert_underline=None):
+def convert_document_element_to_html(element,
+        style_map=None,
+        convert_image=None,
+        convert_underline=None,
+        generate_uniquifier=None):
+            
     if style_map is None:
         style_map = []
+    
+    if generate_uniquifier is None:
+        generate_uniquifier = lambda: random.randint(0, 1000000000000000)
+        
     html_generator = HtmlGenerator()
-    converter = DocumentConverter(style_map, convert_image=convert_image, convert_underline=convert_underline)
+    converter = DocumentConverter(style_map,
+        convert_image=convert_image,
+        convert_underline=convert_underline,
+        generate_uniquifier=generate_uniquifier)
     converter.convert_element_to_html(element, html_generator,)
     html_generator.end_all()
     return results.Result(html_generator.html_string(), converter.messages)
 
 
 class DocumentConverter(object):
-    def __init__(self, style_map, convert_image, convert_underline):
+    def __init__(self, style_map, convert_image, convert_underline, generate_uniquifier):
         self.messages = []
         self._style_map = style_map
+        self._uniquifier = generate_uniquifier()
+        self._footnote_ids = []
         self._converters = {
             documents.Document: self._convert_document,
             documents.Paragraph: self._convert_paragraph,
@@ -30,6 +48,8 @@ class DocumentConverter(object):
             documents.TableCell: self._convert_table_cell,
             documents.LineBreak: self._line_break,
             documents.Image: convert_image or images.inline(self._convert_image),
+            documents.FootnoteReference: self._convert_footnote_reference,
+            documents.Footnote: self._convert_footnote,
         }
         self._convert_underline = convert_underline
 
@@ -40,6 +60,14 @@ class DocumentConverter(object):
 
     def _convert_document(self, document, html_generator):
         self._convert_elements_to_html(document.children, html_generator)
+        html_generator.end_all()
+        html_generator.start("ol")
+        footnotes = [
+            document.footnotes.find_footnote_by_id(footnote_id)
+            for footnote_id in self._footnote_ids
+        ]
+        self._convert_elements_to_html(footnotes, html_generator)
+        html_generator.end()
 
 
     def _convert_paragraph(self, paragraph, html_generator):
@@ -113,6 +141,31 @@ class DocumentConverter(object):
         return {
             "src": "data:{0};base64,{1}".format(image.content_type, encoded_src)
         }
+    
+    def _convert_footnote_reference(self, footnote_reference, html_generator):
+        uid = self._footnote_uid(footnote_reference.footnote_id)
+        html_generator.start("sup")
+        html_generator.start("a", {
+            "href": "#footnote-" + uid,
+            "id": "footnote-ref-" + uid
+        })
+        self._footnote_ids.append(footnote_reference.footnote_id);
+        footnote_number = len(self._footnote_ids)
+        html_generator.text("[{0}]".format(footnote_number))
+        html_generator.end()
+        html_generator.end()
+    
+    def _convert_footnote(self, footnote, html_generator):
+        uid = self._footnote_uid(footnote.id)
+        html_generator.start("li", {"id": "footnote-{0}".format(uid)})
+        footnote_generator = HtmlGenerator()
+        self._convert_elements_to_html(footnote.body, footnote_generator)
+        footnote_generator.text(" ")
+        footnote_generator.start("a", {"href": "#footnote-ref-{0}".format(uid)})
+        footnote_generator.text(_up_arrow)
+        footnote_generator.end_all()
+        html_generator.append(footnote_generator)
+        html_generator.end()
 
 
     def _convert_elements_to_html(self, elements, html_generator):
@@ -143,6 +196,10 @@ class DocumentConverter(object):
         return default
         
 
+    def _footnote_uid(self, footnote_id):
+        return "{0}-{1}".format(self._uniquifier, footnote_id)
+        
+
 def _document_matcher_matches(matcher, element, element_type):
     return (
         matcher.element_type == element_type and (
@@ -157,3 +214,7 @@ def _document_matcher_matches(matcher, element, element_type):
             matcher.numbering == element.numbering
         )
     )
+
+_up_arrow = "↑"
+if sys.version_info[0] < 3:
+    _up_arrow = _up_arrow.decode("utf8")
