@@ -17,7 +17,10 @@ def _collect(values, result):
 
 if sys.version_info[0] == 2:
     map = map
+    filter = filter
 else:
     import builtins
     def map(*args, **kwargs):
         return list(builtins.map(*args, **kwargs))
+    def filter(*args, **kwargs):
+        return list(builtins.filter(*args, **kwargs))
